@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: arg_check.c,v 1.29 2000/11/13 15:47:18 gray Exp $
+ * $Id: arg_check.c,v 1.33 2003/05/15 02:42:14 gray Exp $
  */
 
 /*
@@ -37,20 +37,11 @@
 #include "conf.h"
 
 #include "chunk.h"
-#include "debug_val.h"
+#include "debug_tok.h"
 #include "error.h"
 #include "dmalloc_loc.h"
 #include "arg_check.h"
 
-#if INCLUDE_RCS_IDS
-#if IDENT_WORKS
-#ident "$Id: arg_check.c,v 1.29 2000/11/13 15:47:18 gray Exp $"
-#else
-static	char	*rcs_id =
-  "$Id: arg_check.c,v 1.29 2000/11/13 15:47:18 gray Exp $";
-#endif
-#endif
-
 #if HAVE_BCMP
 /*
  * Dummy function for checking bcmp's arguments.
@@ -58,9 +49,9 @@ static	char	*rcs_id =
 int	_dmalloc_bcmp(const void *b1, const void *b2, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("bcmp", b1, CHUNK_PNT_LOOSE, len))
-	|| (! _chunk_pnt_check("bcmp", b2, CHUNK_PNT_LOOSE, len))) {
-      _dmalloc_message("bad pointer argument found in bcmp");
+    if ((! _dmalloc_chunk_pnt_check("bcmp", b1, 0, len))
+	|| (! _dmalloc_chunk_pnt_check("bcmp", b2, 0, len))) {
+      dmalloc_message("bad pointer argument found in bcmp");
     }
   }
   return bcmp(b1, b2, len);
@@ -74,9 +65,9 @@ int	_dmalloc_bcmp(const void *b1, const void *b2, const DMALLOC_SIZE len)
 void	_dmalloc_bcopy(const void *from, void *to, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("bcopy", from, CHUNK_PNT_LOOSE, len))
-	|| (! _chunk_pnt_check("bcopy", to, CHUNK_PNT_LOOSE, len))) {
-      _dmalloc_message("bad pointer argument found in bcopy");
+    if ((! _dmalloc_chunk_pnt_check("bcopy", from, 0, len))
+	|| (! _dmalloc_chunk_pnt_check("bcopy", to, 0, len))) {
+      dmalloc_message("bad pointer argument found in bcopy");
     }
   }
   bcopy(from, to, len);
@@ -90,9 +81,9 @@ void	_dmalloc_bcopy(const void *from, void *to, const DMALLOC_SIZE len)
 int	_dmalloc_memcmp(const void *b1, const void *b2, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("memcmp", b1, CHUNK_PNT_LOOSE, len))
-	|| (! _chunk_pnt_check("memcmp", b2, CHUNK_PNT_LOOSE, len))) {
-      _dmalloc_message("bad pointer argument found in memcmp");
+    if ((! _dmalloc_chunk_pnt_check("memcmp", b1, 0, len))
+	|| (! _dmalloc_chunk_pnt_check("memcmp", b2, 0, len))) {
+      dmalloc_message("bad pointer argument found in memcmp");
     }
   }
   return memcmp(b1, b2, len);
@@ -106,9 +97,9 @@ int	_dmalloc_memcmp(const void *b1, const void *b2, const DMALLOC_SIZE len)
 void	*_dmalloc_memcpy(void *to, const void *from, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("memcpy", to, CHUNK_PNT_LOOSE, len))
-	|| (! _chunk_pnt_check("memcpy", from, CHUNK_PNT_LOOSE, len))) {
-      _dmalloc_message("bad pointer argument found in memcpy");
+    if ((! _dmalloc_chunk_pnt_check("memcpy", to, 0, len))
+	|| (! _dmalloc_chunk_pnt_check("memcpy", from, 0, len))) {
+      dmalloc_message("bad pointer argument found in memcpy");
     }
   }
   return (void *)memcpy(to, from, len);
@@ -122,8 +113,8 @@ void	*_dmalloc_memcpy(void *to, const void *from, const DMALLOC_SIZE len)
 void	*_dmalloc_memset(void *buf, const int ch, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("memset", buf, CHUNK_PNT_LOOSE, len)) {
-      _dmalloc_message("bad pointer argument found in memset");
+    if (! _dmalloc_chunk_pnt_check("memset", buf, 0, len)) {
+      dmalloc_message("bad pointer argument found in memset");
     }
   }
   return (void *)memset(buf, ch, len);
@@ -137,9 +128,8 @@ void	*_dmalloc_memset(void *buf, const int ch, const DMALLOC_SIZE len)
 char	*_dmalloc_index(const char *str, const char ch)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("index", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			   0)) {
-      _dmalloc_message("bad pointer argument found in index");
+    if (! _dmalloc_chunk_pnt_check("index", str, 0, -1)) {
+      dmalloc_message("bad pointer argument found in index");
     }
   }
   return (char *)index(str, ch);
@@ -153,9 +143,8 @@ char	*_dmalloc_index(const char *str, const char ch)
 char	*_dmalloc_rindex(const char *str, const char ch)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("rindex", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			   0)) {
-      _dmalloc_message("bad pointer argument found in rindex");
+    if (! _dmalloc_chunk_pnt_check("rindex", str, 0, -1)) {
+      dmalloc_message("bad pointer argument found in rindex");
     }
   }
   return (char *)rindex(str, ch);
@@ -169,11 +158,10 @@ char	*_dmalloc_rindex(const char *str, const char ch)
 char	*_dmalloc_strcat(char *to, const char *from)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strcat", to, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    strlen(to) + strlen(from) + 1))
-	|| (! _chunk_pnt_check("strcat", from,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strcat");
+    if ((! _dmalloc_chunk_pnt_check("strcat", to, 0,
+				    strlen(to) + strlen(from) + 1))
+	|| (! _dmalloc_chunk_pnt_check("strcat", from, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strcat");
     }
   }
   return (char *)strcat(to, from);
@@ -187,11 +175,9 @@ char	*_dmalloc_strcat(char *to, const char *from)
 int	_dmalloc_strcmp(const char *s1, const char *s2)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strcmp", s1, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strcmp", s2, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			       0))) {
-      _dmalloc_message("bad pointer argument found in strcmp");
+    if ((! _dmalloc_chunk_pnt_check("strcmp", s1, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strcmp", s2, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strcmp");
     }
   }
   return strcmp(s1, s2);
@@ -205,9 +191,8 @@ int	_dmalloc_strcmp(const char *s1, const char *s2)
 DMALLOC_SIZE	_dmalloc_strlen(const char *str)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("strlen", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			   0)) {
-      _dmalloc_message("bad pointer argument found in strlen");
+    if (! _dmalloc_chunk_pnt_check("strlen", str, 0, -1)) {
+      dmalloc_message("bad pointer argument found in strlen");
     }
   }
   return strlen(str);
@@ -222,11 +207,9 @@ char	*_dmalloc_strtok(char *str, const char *sep)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
     if ((str != NULL
-	 && (! _chunk_pnt_check("strtok", str,
-				CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0)))
-	|| (! _chunk_pnt_check("strtok", sep,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strtok");
+	 && (! _dmalloc_chunk_pnt_check("strtok", str, 0, -1)))
+	|| (! _dmalloc_chunk_pnt_check("strtok", sep, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strtok");
     }
   }
   return (char *)strtok(str, sep);
@@ -240,8 +223,8 @@ char	*_dmalloc_strtok(char *str, const char *sep)
 void	_dmalloc_bzero(void *buf, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("bzero", buf, CHUNK_PNT_LOOSE, len)) {
-      _dmalloc_message("bad pointer argument found in bzero");
+    if (! _dmalloc_chunk_pnt_check("bzero", buf, 0, len)) {
+      dmalloc_message("bad pointer argument found in bzero");
     }
   }
   bzero(buf, len);
@@ -257,9 +240,9 @@ void	*_dmalloc_memccpy(void *s1, const void *s2, const int ch,
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
     /* maybe len maybe first ch */
-    if ((! _chunk_pnt_check("memccpy", s1, CHUNK_PNT_LOOSE, 0))
-	|| (! _chunk_pnt_check("memccpy", s2, CHUNK_PNT_LOOSE, 0))) {
-      _dmalloc_message("bad pointer argument found in memccpy");
+    if ((! _dmalloc_chunk_pnt_check("memccpy", s1, 0, 0))
+	|| (! _dmalloc_chunk_pnt_check("memccpy", s2, 0, 0))) {
+      dmalloc_message("bad pointer argument found in memccpy");
     }
   }
   return (void *)memccpy(s1, s2, ch, len);
@@ -273,8 +256,8 @@ void	*_dmalloc_memccpy(void *s1, const void *s2, const int ch,
 void	*_dmalloc_memchr(const void *s1, const int ch, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("memchr", s1, CHUNK_PNT_LOOSE, len)) {
-      _dmalloc_message("bad pointer argument found in memchr");
+    if (! _dmalloc_chunk_pnt_check("memchr", s1, 0, len)) {
+      dmalloc_message("bad pointer argument found in memchr");
     }
   }
   return (void *)memchr(s1, ch, len);
@@ -288,9 +271,8 @@ void	*_dmalloc_memchr(const void *s1, const int ch, const DMALLOC_SIZE len)
 char	*_dmalloc_strchr(const char *str, const int ch)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("strchr", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			   0)) {
-      _dmalloc_message("bad pointer argument found in strchr");
+    if (! _dmalloc_chunk_pnt_check("strchr", str, 0, -1)) {
+      dmalloc_message("bad pointer argument found in strchr");
     }
   }
   return (char *)strchr(str, ch);
@@ -304,9 +286,8 @@ char	*_dmalloc_strchr(const char *str, const int ch)
 char	*_dmalloc_strrchr(const char *str, const int ch)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if (! _chunk_pnt_check("strrchr", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			   0)) {
-      _dmalloc_message("bad pointer argument found in strrchr");
+    if (! _dmalloc_chunk_pnt_check("strrchr", str, 0, -1)) {
+      dmalloc_message("bad pointer argument found in strrchr");
     }
   }
   return (char *)strrchr(str, ch);
@@ -320,11 +301,9 @@ char	*_dmalloc_strrchr(const char *str, const int ch)
 char	*_dmalloc_strcpy(char *to, const char *from)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strcpy", to, CHUNK_PNT_LOOSE,
-			    strlen(from) + 1))
-	|| (! _chunk_pnt_check("strcpy", from,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strcpy");
+    if ((! _dmalloc_chunk_pnt_check("strcpy", to, 0, strlen(from) + 1))
+	|| (! _dmalloc_chunk_pnt_check("strcpy", from, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strcpy");
     }
   }
   return (char *)strcpy(to, from);
@@ -339,10 +318,9 @@ char	*_dmalloc_strncpy(char *to, const char *from, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
     /* len or until nullc */
-    if ((! _chunk_pnt_check("strncpy", to, CHUNK_PNT_LOOSE, 0))
-	|| (! _chunk_pnt_check("strncpy", from,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strncpy");
+    if ((! _dmalloc_chunk_pnt_check("strncpy", to, 0, 0))
+	|| (! _dmalloc_chunk_pnt_check("strncpy", from, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strncpy");
     }
   }
   return (char *)strncpy(to, from, len);
@@ -356,11 +334,9 @@ char	*_dmalloc_strncpy(char *to, const char *from, const DMALLOC_SIZE len)
 int	_dmalloc_strcasecmp(const char *s1, const char *s2)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strcasecmp", s1,
-			    CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))
-	|| (! _chunk_pnt_check("strcasecmp", s2,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strcasecmp");
+    if ((! _dmalloc_chunk_pnt_check("strcasecmp", s1, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strcasecmp", s2, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strcasecmp");
     }
   }
   return strcasecmp(s1, s2);
@@ -376,11 +352,9 @@ int	_dmalloc_strncasecmp(const char *s1, const char *s2,
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
     /* len or until nullc */
-    if ((! _chunk_pnt_check("strncasecmp", s1,
-			    CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))
-	|| (! _chunk_pnt_check("strncasecmp", s2,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strncasecmp");
+    if ((! _dmalloc_chunk_pnt_check("strncasecmp", s1, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strncasecmp", s2, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strncasecmp");
     }
   }
   return strncasecmp(s1, s2, len);
@@ -394,11 +368,9 @@ int	_dmalloc_strncasecmp(const char *s1, const char *s2,
 int	_dmalloc_strspn(const char *str, const char *list)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strspn", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strspn", list,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strspn");
+    if ((! _dmalloc_chunk_pnt_check("strspn", str, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strspn", list, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strspn");
     }
   }
   return strspn(str, list);
@@ -412,11 +384,9 @@ int	_dmalloc_strspn(const char *str, const char *list)
 int	_dmalloc_strcspn(const char *str, const char *list)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strcspn", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strcspn", list,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strcspn");
+    if ((! _dmalloc_chunk_pnt_check("strcspn", str, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strcspn", list, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strcspn");
     }
   }
   return strcspn(str, list);
@@ -431,11 +401,9 @@ char	*_dmalloc_strncat(char *to, const char *from, const DMALLOC_SIZE len)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
     /* either len or nullc */
-    if ((! _chunk_pnt_check("strncat", to, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strncat", from,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strncat");
+    if ((! _dmalloc_chunk_pnt_check("strncat", to, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strncat", from, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strncat");
     }
   }
   return (char *)strncat(to, from, len);
@@ -451,11 +419,9 @@ int	_dmalloc_strncmp(const char *s1, const char *s2,
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
     /* either len or nullc */
-    if ((! _chunk_pnt_check("strncmp", s1, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strncmp", s2,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strncmp");
+    if ((! _dmalloc_chunk_pnt_check("strncmp", s1, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strncmp", s2, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strncmp");
     }
   }
   return strncmp(s1, s2, len);
@@ -469,11 +435,9 @@ int	_dmalloc_strncmp(const char *s1, const char *s2,
 char	*_dmalloc_strpbrk(const char *str, const char *list)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strpbrk", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strpbrk", list,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strpbrk");
+    if ((! _dmalloc_chunk_pnt_check("strpbrk", str, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strpbrk", list, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strpbrk");
     }
   }
   return (char *)strpbrk(str, list);
@@ -487,11 +451,9 @@ char	*_dmalloc_strpbrk(const char *str, const char *list)
 char	*_dmalloc_strstr(const char *str, const char *pat)
 {
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_FUNCS)) {
-    if ((! _chunk_pnt_check("strstr", str, CHUNK_PNT_LOOSE | CHUNK_PNT_NULL,
-			    0))
-	|| (! _chunk_pnt_check("strstr", pat,
-			       CHUNK_PNT_LOOSE | CHUNK_PNT_NULL, 0))) {
-      _dmalloc_message("bad pointer argument found in strstr");
+    if ((! _dmalloc_chunk_pnt_check("strstr", str, 0, -1))
+	|| (! _dmalloc_chunk_pnt_check("strstr", pat, 0, -1))) {
+      dmalloc_message("bad pointer argument found in strstr");
     }
   }
   return (char *)strstr(str, pat);
