@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: dmalloc.c,v 1.99 2000/11/13 15:47:17 gray Exp $
+ * $Id: dmalloc.c,v 1.110 2003/09/06 15:12:44 gray Exp $
  */
 
 /*
@@ -50,22 +50,12 @@
 
 #include "compat.h"
 #include "debug_tok.h"
-#include "debug_val.h"
+#include "debug_tok.h"
 #include "env.h"
-#include "error_str.h"
 #include "error_val.h"
 #include "dmalloc_loc.h"
 #include "version.h"
 
-#if INCLUDE_RCS_IDS
-#if IDENT_WORKS
-#ident "$Id: dmalloc.c,v 1.99 2000/11/13 15:47:17 gray Exp $"
-#else
-static	char	*rcs_id =
-  "$Id: dmalloc.c,v 1.99 2000/11/13 15:47:17 gray Exp $";
-#endif
-#endif
-
 #define HOME_ENVIRON	"HOME"			/* home directory */
 #define SHELL_ENVIRON	"SHELL"			/* for the type of shell */
 #define DEFAULT_CONFIG	".dmallocrc"		/* default config file */
@@ -75,6 +65,7 @@ static	char	*rcs_id =
 #define DEBUG_ARG		'd'		/* debug argument */
 #define INTERVAL_ARG		'i'		/* interval argument */
 #define THREAD_LOCK_ON_ARG	'o'		/* lock-on argument */
+#define LIMIT_ARG		'M'		/* memory-limit argument */
 #define LINE_WIDTH		75		/* num debug toks per line */
 
 #define FILE_NOT_FOUND		1
@@ -98,15 +89,14 @@ typedef struct {
 			 DEBUG_FREE_BLANK | DEBUG_ERROR_ABORT | \
 			 DEBUG_ALLOC_BLANK)
 #define MEDIUM_FLAGS	(LOW_FLAGS | \
-			 DEBUG_CHECK_HEAP | DEBUG_CHECK_LISTS | \
+			 DEBUG_CHECK_HEAP | \
 			 DEBUG_REALLOC_COPY)
 #define HIGH_FLAGS	(MEDIUM_FLAGS | \
 			 DEBUG_CHECK_BLANK | DEBUG_CHECK_FUNCS)
 #define ALL_FLAGS	(HIGH_FLAGS | \
 			 DEBUG_LOG_TRANS | DEBUG_LOG_ADMIN | \
-			 DEBUG_LOG_BLOCKS | \
-			 DEBUG_HEAP_CHECK_MAP | DEBUG_NEVER_REUSE)
-/* NOTE: print-error is not in this list because it is special */
+			 DEBUG_NEVER_REUSE)
+/* NOTE: print-messages is not in this list because it is special */
 
 static	default_t	defaults[] = {
   { "none",		0 },
@@ -138,11 +128,12 @@ static	int	debug_tokens_b = 0;		/* list debug tokens */
 static	char	*logpath = NULL;		/* for LOGFILE setting */
 static	int	long_tokens_b = 0;		/* long-tok output */
 static	argv_array_t	minus;			/* tokens to remove */
+static	unsigned long limit_arg = 0;		/* memory limit */
 static	int	make_changes_b = 1;		/* make no changes to env */
 static	argv_array_t	plus;			/* tokens to add */
 static	int	remove_auto_b = 0;		/* auto-remove settings */
-static	int	short_tokens_b = 0;		/* short-tok output */
 static	char	*start = NULL;			/* for START settings */
+static	int	usage_b = 0;			/* usage messages */
 static	int	verbose_b = 0;			/* verbose flag */
 static	int	very_verbose_b = 0;		/* very-verbose flag */
 static	char	*tag = NULL;			/* maybe a tag argument */
@@ -159,9 +150,6 @@ static	argv_t	args[] = {
   
   { 'L',	"long-tokens",	ARGV_BOOL_INT,	&long_tokens_b,
     NULL,			"output long-tokens not 0x..." },
-  { ARGV_OR },
-  { 'S',	"short-tokens",	ARGV_BOOL_INT,	&short_tokens_b,
-    NULL,			"output short-tokens not 0x..." },
   
   { 'a',	"address",	ARGV_CHAR_P,	&address,
     "address:#",		"stop when malloc sees address" },
@@ -174,7 +162,7 @@ static	argv_t	args[] = {
   { 'e',	"errno",	ARGV_INT,	&errno_to_print,
     "errno",			"print error string for errno" },
   { 'f',	"file",		ARGV_CHAR_P,	&inpath,
-    "path",			"config if not ~/.mallocrc" },
+    "path",			"config if not $HOME/.dmallocrc" },
   { 'h',	"help",		ARGV_BOOL_INT,	&help_b,
     NULL,			"print help message" },
   { INTERVAL_ARG, "interval",	ARGV_U_LONG,	&interval,
@@ -185,6 +173,8 @@ static	argv_t	args[] = {
     "path",			"file to log messages to" },
   { 'm',	"minus",	ARGV_CHAR_P | ARGV_FLAG_ARRAY,	&minus,
     "token(s)",			"del tokens from current debug" },
+  { LIMIT_ARG,	"memory-limit",	ARGV_U_SIZE,	&limit_arg,
+    "value",			"limit allocations to this amount" },
   { 'n',	"no-changes",	ARGV_BOOL_NEG,	&make_changes_b,
     NULL,			"make no changes to the env" },
   { THREAD_LOCK_ON_ARG, "lock-on", ARGV_INT,	&thread_lock_on,
@@ -197,6 +187,8 @@ static	argv_t	args[] = {
     "file:line",		"start check heap after this" },
   { 't',	"list-tags",	ARGV_BOOL_INT,	&list_tags_b,
     NULL,			"list tags in rc file" },
+  { 'u',	"usage",	ARGV_BOOL_INT,	&usage_b,
+    NULL,			"print usage messages" },
   { 'v',	"verbose",	ARGV_BOOL_INT,	&verbose_b,
     NULL,			"turn on verbose output" },
   { 'V',	"very-verbose",	ARGV_BOOL_INT,	&very_verbose_b,
@@ -267,18 +259,12 @@ static	void	dump_debug(const unsigned long val)
       }
       
       if (very_verbose_b) {
-	(void)fprintf(stderr, "%s (%s) -- %s (%#lx)\n",
-		      attr_p->at_string, attr_p->at_short, attr_p->at_desc,
-		      attr_p->at_value);
+	(void)fprintf(stderr, "%s -- %s (%#lx)\n",
+		      attr_p->at_string, attr_p->at_desc, attr_p->at_value);
 	col_c = 0;
       }
       else {
-	if (short_tokens_b) {
-	  str = attr_p->at_short;
-	}
-	else {
-	  str = attr_p->at_string;
-	}
+	str = attr_p->at_string;
 	len = strlen(str);
 	if (col_c + len + 2 > LINE_WIDTH) {
 	  (void)fprintf(stderr, "\n");
@@ -320,8 +306,7 @@ static	long	token_to_value(const char *tok)
   
   /* find the matching attribute string */
   for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
-    if (strcmp(tok, attr_p->at_string) == 0
-	|| strcmp(tok, attr_p->at_short) == 0) {
+    if (strcmp(tok, attr_p->at_string) == 0) {
       break;
     }
   }
@@ -468,8 +453,8 @@ static	int	read_rc_file(const char *path, const long debug_value,
   
   SET_POINTER(debug_p, new_debug);
   if (token != NULL) {
-    strncpy(token, next_token, token_size);
-    token[token_size - 1] = '\0';
+    (void)loc_snprintf(token, token_size, "config file token: %s",
+		       next_token);
   }
   
   if (found_b) {
@@ -486,8 +471,8 @@ static	int	read_rc_file(const char *path, const long debug_value,
  * into TOKEN of TOKEN_SIZE.  Routine returns the new debug value
  * matching tag.
  */
-static	long	process(const long debug_value, const char *tag_find,
-			char *token, const int token_size)
+static	long	find_tag(const long debug_value, const char *tag_find,
+			 char *token, const int token_size)
 {
   char		path[1024], *path_p;
   default_t	*def_p;
@@ -561,15 +546,14 @@ static	long	process(const long debug_value, const char *tag_find,
     if (token != NULL) {
       for (def_p = defaults; def_p->de_string != NULL; def_p++) {
 	if (def_p->de_flags == debug_value) {
-	  strncpy(token, def_p->de_string, token_size);
-	  token[token_size - 1] = '\0';
+	  (void)loc_snprintf(token, token_size, "internal token: %s",
+			     def_p->de_string);
 	  new_debug = def_p->de_flags;
 	  break;
 	}
       }
       if (def_p->de_string == NULL) {
-	strncpy(token, "unknown", token_size);
-	token[token_size - 1] = '\0';
+	(void)loc_snprintf(token, token_size, "unknown token");
 	new_debug = 0;
       }
     }
@@ -606,7 +590,7 @@ static	long	process(const long debug_value, const char *tag_find,
  */
 static	void	list_tags(void)
 {
-  char		path[1024], *path_p, token[64];
+  char		path[1024], *path_p, token[80];
   default_t	*def_p;
   const char	*home_p;
   long		new_debug = 0;
@@ -688,26 +672,27 @@ static	void	list_tags(void)
 static	void	dump_current(void)
 {
   char		*lpath, *start_file, token[64];
+  const char	*env_str;
   DMALLOC_PNT	addr;
-  unsigned long	inter;
+  unsigned long	inter, limit_val;
   long		addr_count;
   int		lock_on, start_line, start_count;
   unsigned int	flags;
   
-  (void)fprintf(stderr, "Debug Malloc Utility: http://dmalloc.com/\n");
-  (void)fprintf(stderr,
-		"  For a list of the command-line options enter: %s --usage\n",
-		argv_argv[0]);
-  
-  _dmalloc_environ_get(OPTIONS_ENVIRON, &addr, &addr_count, &flags,
-		       &inter, &lock_on, &lpath,
-		       &start_file, &start_line, &start_count);
+  /* get the options flag */
+  env_str = getenv(OPTIONS_ENVIRON);
+  if (env_str == NULL) {
+    env_str = "";
+  }
+  _dmalloc_environ_process(env_str, &addr, &addr_count, &flags,
+			   &inter, &lock_on, &lpath,
+			   &start_file, &start_line, &start_count, &limit_val);
   
   if (flags == 0) {
     (void)fprintf(stderr, "Debug-Flags  not-set\n");
   }
   else {
-    (void)process(flags, NULL, token, sizeof(token));
+    (void)find_tag(flags, NULL, token, sizeof(token));
     (void)fprintf(stderr, "Debug-Flags %#x (%u) (%s)\n",
 		  flags, flags, token);
     if (verbose_b) {
@@ -749,6 +734,13 @@ static	void	dump_current(void)
     (void)fprintf(stderr, "Logpath      '%s'\n", lpath);
   }
   
+  if (limit_val == 0) {
+    (void)fprintf(stderr, "Mem-Limit    not-set\n");
+  }
+  else {
+    (void)fprintf(stderr, "Mem-Limit    %lu\n", limit_val);
+  }
+  
   if (start_file == NULL && start_count == 0) {
     (void)fprintf(stderr, "Start-File   not-set\n");
   }
@@ -759,6 +751,12 @@ static	void	dump_current(void)
     (void)fprintf(stderr, "Start-File   '%s', line = %d\n",
 		  start_file, start_line);
   }
+  
+  (void)fprintf(stderr, "\n");
+  (void)fprintf(stderr, "Debug Malloc Utility: http://dmalloc.com/\n");
+  (void)fprintf(stderr,
+		"  For a list of the command-line options enter: %s --usage\n",
+		argv_argv[0]);
 }
 
 /*
@@ -768,7 +766,10 @@ static	void    set_variable(const char *var, const char *value)
 {
   char	comm[1024];
   
-  if (bourne_b) {
+  if (value == NULL || *value == '\0') {
+    (void)loc_snprintf(comm, sizeof(comm), "unset %s\n", var);
+  }
+  else if (bourne_b) {
     (void)loc_snprintf(comm, sizeof(comm), "%s=%s\nexport %s\n",
 		       var, value, var);
   }
@@ -804,13 +805,37 @@ static	char	*local_strerror(const int error_num)
   return INVALID_ERROR;
 }
 
+/*
+ * static void header
+ *
+ * DESCRIPTION:
+ *
+ * Print out a little usage header to the user.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * None.
+ */
+static	void	header(void)
+{
+  (void)fprintf(stderr,
+		"Debug Malloc Utility: http://dmalloc.com/\n");
+  (void)fprintf(stderr,
+		"  This utility helps set the Debug Malloc environment variables.\n");
+}
+
 int	main(int argc, char **argv)
 {
   char		buf[1024];
   int		set_b = 0;
   char		*lpath, *sfile;
+  const char	*env_str;
   DMALLOC_PNT	addr;
-  unsigned long	inter;
+  unsigned long	inter, limit_val;
   long		addr_count;
   int		lock_on;
   int		sline, scount;
@@ -822,14 +847,17 @@ int	main(int argc, char **argv)
   argv_process(args, argc, argv);
   
   if (help_b) {
-    (void)fprintf(stderr, "Debug Malloc Utility: http://dmalloc.com/\n");
-    (void)fprintf(stderr,
-		  "  This utility helps set the Debug Malloc environment variables.\n");
+    header();
     (void)fprintf(stderr,
 		  "  For a list of the command-line options enter: %s --usage\n",
 		  argv_argv[0]);
     exit(0);
   }
+  if (usage_b) {
+    header();
+    argv_usage(args, ARGV_USAGE_ALL);
+    exit(0);
+  }
   
   if (very_verbose_b) {
     verbose_b = 1;
@@ -841,8 +869,13 @@ int	main(int argc, char **argv)
   }
   
   /* get the current debug information from the env variable */
-  _dmalloc_environ_get(OPTIONS_ENVIRON, &addr, &addr_count, &flags, &inter,
-		       &lock_on, &lpath, &sfile, &sline, &scount);
+  env_str = getenv(OPTIONS_ENVIRON);
+  if (env_str == NULL) {
+    env_str = "";
+  }
+  _dmalloc_environ_process(env_str, &addr, &addr_count, &flags, &inter,
+			   &lock_on, &lpath, &sfile, &sline, &scount,
+			   &limit_val);
   
   /*
    * So, if a tag was specified on the command line then we set the
@@ -868,7 +901,7 @@ int	main(int argc, char **argv)
 		    argv_program, tag);
     }
     set_b = 1;
-    debug = process(0L, tag, NULL, 0);
+    debug = find_tag(0L, tag, NULL, 0);
     /* should we clear the rest? */
     if (remove_auto_b && (! keep_b)) {
       clear_b = 1;
@@ -878,7 +911,7 @@ int	main(int argc, char **argv)
   if (plus.aa_entry_n > 0) {
     int		plus_c;
     for (plus_c = 0; plus_c < plus.aa_entry_n; plus_c++) {
-      debug |= token_to_value(ARGV_ARRAY_ENTRY(plus, char *, plus_c));
+      BIT_SET(debug, token_to_value(ARGV_ARRAY_ENTRY(plus, char *, plus_c)));
       set_b = 1;
     }
   }
@@ -886,7 +919,8 @@ int	main(int argc, char **argv)
   if (minus.aa_entry_n > 0) {
     int		minus_c;
     for (minus_c = 0; minus_c < minus.aa_entry_n; minus_c++) {
-      debug &= ~token_to_value(ARGV_ARRAY_ENTRY(minus, char *, minus_c));
+      BIT_CLEAR(debug,
+		token_to_value(ARGV_ARRAY_ENTRY(minus, char *, minus_c)));
       set_b = 1;
     }
   }
@@ -940,6 +974,11 @@ int	main(int argc, char **argv)
     scount = 0;
   }
   
+  if (argv_was_used(args, LIMIT_ARG)) {
+    limit_val = limit_arg;
+    set_b = 1;
+  }
+  
   if (errno_to_print > 0) {
     (void)fprintf(stderr, "%s: dmalloc_errno value '%d' = \n",
 		  argv_program, errno_to_print);
@@ -965,14 +1004,12 @@ int	main(int argc, char **argv)
 	continue;
       }
       if (very_verbose_b) {
-	(void)fprintf(stderr, "%s (%s) -- %s (%#lx)\n",
-		      attr_p->at_string, attr_p->at_short, attr_p->at_desc,
-		      attr_p->at_value);
+	(void)fprintf(stderr, "%s -- %s (%#lx)\n",
+		      attr_p->at_string, attr_p->at_desc, attr_p->at_value);
       }
       else if (verbose_b) {
 	(void)fprintf(stderr, "%s -- %s\n",
-		      (short_tokens_b ? attr_p->at_short : attr_p->at_string),
-		      attr_p->at_desc);
+		      attr_p->at_string, attr_p->at_desc);
       }
       else {
 	(void)fprintf(stderr, "%s\n", attr_p->at_string);
@@ -982,9 +1019,9 @@ int	main(int argc, char **argv)
   }
   
   if (clear_b || set_b) {
-    _dmalloc_environ_set(buf, sizeof(buf), long_tokens_b, short_tokens_b,
-			 addr, addr_count, debug, inter, lock_on, lpath,
-			 sfile, sline, scount);
+    _dmalloc_environ_set(buf, sizeof(buf), long_tokens_b, addr, addr_count,
+			 debug, inter, lock_on, lpath, sfile, sline, scount,
+			 limit_val);
     set_variable(OPTIONS_ENVIRON, buf);
   }
   else if (errno_to_print == 0
