@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: env.c,v 1.27 2000/11/13 15:46:52 gray Exp $
+ * $Id: env.c,v 1.34 2003/06/10 00:25:00 gray Exp $
  */
 
 /*
@@ -47,14 +47,21 @@
 #include "env.h"
 #include "error.h"
 
-#if INCLUDE_RCS_IDS
-#if IDENT_WORKS
-#ident "$Id: env.c,v 1.27 2000/11/13 15:46:52 gray Exp $"
-#else
-static	char	*rcs_id =
-  "$Id: env.c,v 1.27 2000/11/13 15:46:52 gray Exp $";
-#endif
-#endif
+/*
+ * Environmental labels.
+ *
+ * NOTE: the decision has been made _not_ to do the sizeof() hack for
+ * portability reasons.
+ */
+#define ADDRESS_LABEL		"addr"
+#define DEBUG_LABEL		"debug"
+#define INTERVAL_LABEL		"inter"
+#define LOCK_ON_LABEL		"lockon"
+#define LOGFILE_LABEL		"log"
+#define START_LABEL		"start"
+#define LIMIT_LABEL		"limit"
+
+#define ASSIGNMENT_CHAR		'='
 
 /* local variables */
 static	char		log_path[512]	= { '\0' }; /* storage for env path */
@@ -125,7 +132,8 @@ void	_dmalloc_start_break(const char *start_all, char **sfile_p,
   
   start_p = strchr(start_all, ':');
   if (start_p != NULL) {
-    (void)strcpy(start_file, start_all);
+    (void)strncpy(start_file, start_all, sizeof(start_file));
+    start_file[sizeof(start_file) - 1] = '\0';
     SET_POINTER(sfile_p, start_file);
     start_p = start_file + (start_p - start_all);
     *start_p = '\0';
@@ -140,13 +148,13 @@ void	_dmalloc_start_break(const char *start_all, char **sfile_p,
  * Process the values of dmalloc environ variable(s) from ENVIRON
  * string.
  */
-void	_dmalloc_environ_get(const char *environ, DMALLOC_PNT *addr_p,
-			     long *addr_count_p, unsigned int *debug_p,
-			     unsigned long *interval_p, int *lock_on_p,
-			     char **logpath_p, char **sfile_p,
-			     int *sline_p, int *scount_p)
+void	_dmalloc_environ_process(const char *env_str, DMALLOC_PNT *addr_p,
+				 long *addr_count_p, unsigned int *debug_p,
+				 unsigned long *interval_p, int *lock_on_p,
+				 char **logpath_p, char **sfile_p,
+				 int *sline_p, int *scount_p,
+				 unsigned long *limit_p)
 {
-  const char	*env;
   char		*env_p, *this_p;
   char		buf[1024];
   int		len, done_b = 0;
@@ -162,15 +170,11 @@ void	_dmalloc_environ_get(const char *environ, DMALLOC_PNT *addr_p,
   SET_POINTER(sfile_p, NULL);
   SET_POINTER(sline_p, 0);
   SET_POINTER(scount_p, 0);
-  
-  /* get the options flag */
-  env = getenv(environ);
-  if (env == NULL) {
-    return;
-  }
+  SET_POINTER(limit_p, 0);
   
   /* make a copy */
-  (void)strcpy(buf, env);
+  (void)strncpy(buf, env_str, sizeof(buf));
+  buf[sizeof(buf) - 1] = '\0';
   
   /* handle each of tokens, in turn */
   for (env_p = buf, this_p = buf; ! done_b; env_p++, this_p = env_p) {
@@ -231,18 +235,9 @@ void	_dmalloc_environ_get(const char *environ, DMALLOC_PNT *addr_p,
     if (strncmp(this_p, LOGFILE_LABEL, len) == 0
 	&& *(this_p + len) == ASSIGNMENT_CHAR) {
       this_p += len + 1;
-#if HAVE_GETPID
-      /*
-       * NOTE: this may cause core dumps if this_p contains a bad
-       * format string
-       */
-      (void)loc_snprintf(log_path, sizeof(log_path), this_p, getpid());
-#else
-      (void)strcpy(log_path, this_p);
-#endif
-      if (logpath_p != NULL) {
-	*logpath_p = log_path;
-      }
+      (void)strncpy(log_path, this_p, sizeof(log_path));
+      log_path[sizeof(log_path) - 1] = '\0';
+      SET_POINTER(logpath_p, log_path);
       continue;
     }
     
@@ -258,10 +253,18 @@ void	_dmalloc_environ_get(const char *environ, DMALLOC_PNT *addr_p,
       continue;
     }
     
+    /* set the memory limit to the library */
+    len = strlen(LIMIT_LABEL);
+    if (strncmp(this_p, LIMIT_LABEL, len) == 0
+	&& *(this_p + len) == ASSIGNMENT_CHAR) {
+      this_p += len + 1;
+      SET_POINTER(limit_p, (unsigned long)atol(this_p));
+      continue;
+    }
+    
     /* need to check the short/long debug options */
     for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
-      if (strcmp(this_p, attr_p->at_string) == 0
-	  || strcmp(this_p, attr_p->at_short) == 0) {
+      if (strcmp(this_p, attr_p->at_string) == 0) {
 	flags |= attr_p->at_value;
 	break;
       }
@@ -288,30 +291,24 @@ void	_dmalloc_environ_get(const char *environ, DMALLOC_PNT *addr_p,
  */
 void	_dmalloc_environ_set(char *buf, const int buf_size,
 			     const int long_tokens_b,
-			     const int short_tokens_b,
 			     const DMALLOC_PNT address,
 			     const unsigned long addr_count,
 			     const unsigned int debug,
-			     const int interval, const int lock_on,
+			     const unsigned long interval, const int lock_on,
 			     const char *logpath, const char *start_file_p,
-			     const int start_line, const int start_count)
+			     const int start_line, const int start_count,
+			     const unsigned long limit_val)
 {
   char	*buf_p = buf, *bounds_p = buf + buf_size;
   
   if (debug > 0) {
-    if (short_tokens_b || long_tokens_b) {
+    if (long_tokens_b) {
       attr_t	*attr_p;
       
       for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
 	if (debug & attr_p->at_value) {
-	  if (short_tokens_b) {
-	    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s,",
-				  attr_p->at_short);
-	  }
-	  else {
-	    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s,",
-				  attr_p->at_string);
-	  }
+	  buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s,",
+				attr_p->at_string);
 	}
       }
     }
@@ -332,7 +329,7 @@ void	_dmalloc_environ_set(char *buf, const int buf_size,
     }
   }
   if (interval > 0) {
-    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%d,",
+    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%lu,",
 			  INTERVAL_LABEL, ASSIGNMENT_CHAR, interval);
   }
   if (lock_on > 0) {
@@ -358,6 +355,10 @@ void	_dmalloc_environ_set(char *buf, const int buf_size,
     buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%d,",
 			  START_LABEL, ASSIGNMENT_CHAR, start_count);
   }
+  if (limit_val > 0) {
+    buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s%c%lu,",
+			  LIMIT_LABEL, ASSIGNMENT_CHAR, limit_val);
+  }
   
   /* cut off the last comma */
   if (buf_p > buf) {
