@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: error.c,v 1.96 2000/11/13 15:46:51 gray Exp $
+ * $Id: error.c,v 1.110 2003/09/05 21:29:44 gray Exp $
  */
 
 /*
@@ -35,9 +35,6 @@
 #if HAVE_STDLIB_H
 # include <stdlib.h>				/* for abort */
 #endif
-#if HAVE_STRING_H
-# include <string.h>				/* for strlen */
-#endif
 #if HAVE_UNISTD_H
 # include <unistd.h>				/* for write */
 #endif
@@ -51,7 +48,7 @@
 #endif
 #endif
 
-#if STORE_TIMEVAL
+#if LOG_PNT_TIMEVAL
 # ifdef TIMEVAL_INCLUDE
 #  include TIMEVAL_INCLUDE
 # endif
@@ -67,35 +64,34 @@
 
 #include "dmalloc.h"
 
+#include "chunk.h"				/* for _dmalloc_memory_limit */
 #include "compat.h"
-#include "debug_val.h"
+#include "debug_tok.h"
 #include "env.h"				/* for LOGPATH_INIT */
 #include "error.h"
 #include "error_val.h"
 #include "dmalloc_loc.h"
 #include "version.h"
 
-#if INCLUDE_RCS_IDS
+#if LOCK_THREADS
 #if IDENT_WORKS
-#ident "$Id: error.c,v 1.96 2000/11/13 15:46:51 gray Exp $"
+#ident "@(#) $Information: lock-threads is enabled $"
 #else
-static	char	*rcs_id =
-  "$Id: error.c,v 1.96 2000/11/13 15:46:51 gray Exp $";
+static char *information = "@(#) $Information: lock-threads is enabled $"
 #endif
 #endif
 
-#define SECS_IN_HOUR	(MINS_IN_HOUR * SECS_IN_MIN)
 #define MINS_IN_HOUR	60
 #define SECS_IN_MIN	60
+#define SECS_IN_HOUR	(MINS_IN_HOUR * SECS_IN_MIN)
 
 /* external routines */
-extern	char		*_dmalloc_strerror(const int errnum);
+extern	const char	*dmalloc_strerror(const int errnum);
 
 /*
  * exported variables
  */
-/* logfile for dumping dmalloc info, DMALLOC_LOGFILE env var overrides this */
-char		*_dmalloc_logpath = NULL;
+
 /* address to look for.  when discovered call dmalloc_error() */
 DMALLOC_PNT	_dmalloc_address = NULL;
 /* when to stop at an address */
@@ -110,13 +106,13 @@ unsigned long	_dmalloc_iter_c = 0;
 /* how often to check the heap */
 unsigned long	_dmalloc_check_interval = 0;
 
-#if STORE_TIMEVAL
+#if LOG_PNT_TIMEVAL
 /* overhead information storing when the library started up for elapsed time */
 TIMEVAL_TYPE	_dmalloc_start;
 #endif
 
 /* NOTE: we do the ifdef this way for fillproto */
-#if STORE_TIMEVAL == 0
+#if LOG_PNT_TIMEVAL == 0
 #if HAVE_TIME
 TIME_TYPE	_dmalloc_start = 0;
 #endif
@@ -129,31 +125,161 @@ int		_dmalloc_lock_on = 0;
 int		_dmalloc_aborting_b = 0;
 
 /* local variables */
-static int	outfile_fd = -1;		/* output file descriptor */
+static	int	outfile_fd = -1;		/* output file descriptor */
+/* the following are here to reduce stack overhead */
+static	char	error_str[1024];		/* error string buffer */
+static	char	message_str[1024];		/* message string buffer */
 
 /*
+ * void _dmalloc_open_log
+ *
+ * DESCRIPTION:
+ *
  * Open up our log file and write some version of settings
  * information.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * None.
  */
-void	_dmalloc_open_log(void)
+static	void	build_logfile_path(char *buf, const int buf_len)
 {
-  char	str[1024];
+  char	*bounds_p;
+  char	*path_p, *buf_p, *start_p;
+  int	len;
   
-  /* if it's already open or if we don't have a log file configured */
-  if (outfile_fd >= 0
-      || _dmalloc_logpath == NULL) {
+  if (dmalloc_logpath == NULL) {
+    buf[0] = '\0';
     return;
   }
   
+  buf_p = buf;
+  bounds_p = buf + buf_len;
+  
+  start_p = dmalloc_logpath;
+  for (path_p = dmalloc_logpath; *path_p != '\0'; path_p++) {
+    
+    /* if we don't have to do anything special then just continue */
+    if (*path_p != '%' || *(path_p + 1) == '\0') {
+      if (buf_p < bounds_p) {
+	*buf_p++ = *path_p;
+      }
+      continue;
+    }
+    
+    /* skip over the % */
+    path_p++;
+    
+    /* dump the hostname */
+    if (*path_p == 'h') {
+#if HAVE_GETHOSTNAME
+      char	our_host[128];
+      gethostname(our_host, sizeof(our_host));
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s", our_host);
+#else
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "not-gethostname");
+#endif
+    }
+    /* dump the thread-id */
+    if (*path_p == 'i') {
+#if LOG_PNT_THREAD_ID
+      char		id_str[256];
+      THREAD_TYPE	id;
+      
+      id = THREAD_GET_ID();
+      THREAD_ID_TO_STRING(id_str, sizeof(id_str), id);
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%s", id_str);
+#else
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "no-thread-id");
+#endif
+    }
+    /* dump the pid */
+    if (*path_p == 'p') {
+#if HAVE_GETPID
+      /* we make it long in case it's big and we hope it will promote if not */
+      long	our_pid = getpid();
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%ld", our_pid);
+#else
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "no-getpid");
+#endif
+    }
+    /* dump the time value */
+    if (*path_p == 't') {
+#if HAVE_TIME
+      /* we make time a long here so it will promote */
+      long	now;
+      now = time(NULL);
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%ld", now);
+#else
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "no-time");
+#endif
+    }
+    /* dump the user-id */
+    if (*path_p == 'u') {
+#if HAVE_GETUID
+      /* we make it long in case it's big and we hope it will promote if not */
+      long	our_uid = getuid();
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "%ld", our_uid);
+#else
+      buf_p += loc_snprintf(buf_p, bounds_p - buf_p, "no-uid");
+#endif
+    }
+  }
+  
+  if (buf_p >= bounds_p - 1) {
+    /* NOTE: we can't use dmalloc_message of course so do it the hard way */
+    len = loc_snprintf(error_str, sizeof(error_str),
+		       "debug-malloc library: logfile path too large '%s'\r\n",
+		       dmalloc_logpath);
+    (void)write(STDERR, error_str, len);
+  }
+  
+  *buf_p = '\0';
+}
+
+/*
+ * void _dmalloc_open_log
+ *
+ * DESCRIPTION:
+ *
+ * Open up our log file and write some version of settings
+ * information.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * None.
+ */
+void	_dmalloc_open_log(void)
+{
+  char	log_path[1024];
+  int	len;
+  
+  /* if it's already open or if we don't have a log file configured */
+  if (outfile_fd >= 0
+      || dmalloc_logpath == NULL) {
+    return;
+  }
+  
+  build_logfile_path(log_path, sizeof(log_path));
+  
   /* open our logfile */
-  outfile_fd = open(_dmalloc_logpath, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+  outfile_fd = open(log_path, O_WRONLY | O_CREAT | O_TRUNC, 0666);
   if (outfile_fd < 0) {
-    (void)loc_snprintf(str, sizeof(str),
+    /* NOTE: we can't use dmalloc_message of course so do it the hardway */
+    len = loc_snprintf(error_str, sizeof(error_str),
 		       "debug-malloc library: could not open '%s'\r\n",
-		       _dmalloc_logpath);
-    (void)write(STDERR, str, strlen(str));
+		       log_path);
+    (void)write(STDERR, error_str, len);
     /* disable log_path */
-    _dmalloc_logpath = NULL;
+    dmalloc_logpath = NULL;
     return;
   }
   
@@ -162,47 +288,108 @@ void	_dmalloc_open_log(void)
    * this section of code.
    */
   
-  _dmalloc_message("Dmalloc version '%s' from '%s'",
-		   dmalloc_version, DMALLOC_HOME);
-  _dmalloc_message("flags = %#x, logfile '%s'",
-		   _dmalloc_flags, _dmalloc_logpath);
-  _dmalloc_message("interval = %lu, addr = %#lx, seen # = %ld",
-		   _dmalloc_check_interval,
-		   (unsigned long)_dmalloc_address,
-		   _dmalloc_address_seen_n);
+  dmalloc_message("Dmalloc version '%s' from '%s'",
+		  dmalloc_version, DMALLOC_HOME);
+  dmalloc_message("flags = %#x, logfile '%s'", _dmalloc_flags, log_path);
+  dmalloc_message("interval = %lu, addr = %#lx, seen # = %ld, limit = %ld",
+		  _dmalloc_check_interval, (unsigned long)_dmalloc_address,
+		  _dmalloc_address_seen_n, _dmalloc_memory_limit);
 #if LOCK_THREADS
-  _dmalloc_message("threads enabled, lock-on = %d, lock-init = %d",
-		   _dmalloc_lock_on, THREAD_INIT_LOCK);
+  dmalloc_message("threads enabled, lock-on = %d, lock-init = %d",
+		  _dmalloc_lock_on, THREAD_INIT_LOCK);
 #endif
     
-#if STORE_TIMEVAL
+#if LOG_PNT_TIMEVAL
   {
     char	time_buf[64];
-    _dmalloc_message("starting time = %s",
+    dmalloc_message("starting time = %s",
 		     _dmalloc_ptimeval(&_dmalloc_start, time_buf,
 				       sizeof(time_buf), 0));
   }
 #else
-#if HAVE_TIME /* NOT STORE_TIME */
+#if HAVE_TIME /* NOT LOG_PNT_TIME */
   {
     char	time_buf[64];
-    _dmalloc_message("starting time = %s",
+    dmalloc_message("starting time = %s",
 		     _dmalloc_ptime(&_dmalloc_start, time_buf,
 				    sizeof(time_buf), 0));
   }
 #endif
 #endif
+  
+#if HAVE_GETPID
+  {
+    /* we make it long in case it's big and we hope it will promote if not */
+    long	our_pid = getpid();
+    
+    dmalloc_message("process pid = %ld", our_pid);
+  }
+#endif
 }
 
-#if STORE_TIMEVAL
 /*
- * print the time into local buffer which is returned
+ * void _dmalloc_reopen_log
+ *
+ * DESCRIPTION:
+ *
+ * Re-open our log file which basically calls close() on the
+ * logfile-fd.  If we change the name of the log-file then we will
+ * re-open the file.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * None.
+ */
+void	_dmalloc_reopen_log(void)
+{
+  /* no need to reopen it if it hasn't been reopened yet */
+  if (outfile_fd < 0) {
+    return;
+  }
+  
+  if (dmalloc_logpath == NULL) {
+    dmalloc_message("Closing logfile to be reopened as '%s'",
+		     dmalloc_logpath);
+  }
+  else {
+    dmalloc_message("Closing logfile to not be reopened");
+  }
+  
+  (void)close(outfile_fd);
+  outfile_fd = -1;
+  /* we don't call open here, we'll let the next message do it */
+}
+
+#if LOG_PNT_TIMEVAL
+/*
+ * char *_dmalloc_ptimeval
+ *
+ * DESCRIPTION:
+ *
+ * Print the time into local buffer.
+ *
+ * RETURNS:
+ *
+ * Poiner to the buf argument.
+ *
+ * ARGUMENTS:
+ *
+ * timeval_p -> Pointer to a time value.
+ *
+ * buf -> Internal buffer into which we are writing the time.
+ *
+ * buf_size -> Size of the buffer.
+ *
+ * elapsed_b -> Set to 1 to dump the elapsed instead of global time.
  */
 char	*_dmalloc_ptimeval(const TIMEVAL_TYPE *timeval_p, char *buf,
 			   const int buf_size, const int elapsed_b)
 {
-  unsigned long	hrs, mins, secs;
-  unsigned long	usecs;
+  unsigned long	hrs, mins, secs, usecs;
   
   secs = timeval_p->tv_sec;
   usecs = timeval_p->tv_usec;
@@ -234,9 +421,27 @@ char	*_dmalloc_ptimeval(const TIMEVAL_TYPE *timeval_p, char *buf,
 #endif
 
 /* NOTE: we do the ifdef this way for fillproto */
-#if STORE_TIMEVAL == 0 && HAVE_TIME
+#if LOG_PNT_TIMEVAL == 0 && HAVE_TIME
 /*
- * print the time into local buffer which is returned
+ * char *_dmalloc_ptime
+ *
+ * DESCRIPTION:
+ *
+ * Print the time into local buffer.
+ *
+ * RETURNS:
+ *
+ * Poiner to the buf argument.
+ *
+ * ARGUMENTS:
+ *
+ * time_p -> Pointer to a time value.
+ *
+ * buf -> Internal buffer into which we are writing the time.
+ *
+ * buf_size -> Size of the buffer.
+ *
+ * elapsed_b -> Set to 1 to dump the elapsed instead of global time.
  */
 char	*_dmalloc_ptime(const TIME_TYPE *time_p, char *buf, const int buf_size,
 			const int elapsed_b)
@@ -263,31 +468,81 @@ char	*_dmalloc_ptime(const TIME_TYPE *time_p, char *buf, const int buf_size,
 #endif
 
 /*
- * message writer with vprintf like arguments
+ * void _dmalloc_vmessage
+ *
+ * DESCRIPTION:
+ *
+ * Message writer with vprintf like arguments which adds a line to the
+ * dmalloc logfile.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * format -> Printf-style format statement.
+ *
+ * args -> Already converted pointer to a stdarg list.
  */
 void	_dmalloc_vmessage(const char *format, va_list args)
 {
-  char		str[1024], *str_p, *bounds_p;
-  int		len;
+  char	*str_p, *bounds_p;
+  int	len;
   
-  str_p = str;
-  bounds_p = str_p + sizeof(str);
+  str_p = message_str;
+  bounds_p = str_p + sizeof(message_str);
   
   /* no logpath and no print then no workie */
-  if (_dmalloc_logpath == NULL
+  if (dmalloc_logpath == NULL
       && ! BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_MESSAGES)) {
     return;
   }
   
+#if HAVE_GETPID && LOG_REOPEN
+  if (dmalloc_logpath != NULL) {
+    char	*log_p;
+    
+    /*
+     * This static pid will be checked to make sure it doesn't change.
+     * We make it long in case it's big and we hope it will promote if
+     * not.
+     */
+    static long		current_pid = -1;
+    long		new_pid;
+    
+    new_pid = getpid();
+    if (new_pid != current_pid) {
+      /* NOTE: we need to do this _before_ the reopen otherwise we recurse */
+      current_pid = new_pid;
+      
+      /* if the new pid doesn't match the old one then reopen it */
+      if (current_pid >= 0) {
+	
+	/* this only works if there is a %p in the logpath */
+	for (log_p = dmalloc_logpath; *log_p != '\0'; log_p++) {
+	  if (*log_p == '%' && *(log_p + 1) == 'p') {
+	    _dmalloc_reopen_log();
+	    break;
+	  }
+	}
+      }
+    }
+  }
+#endif
+  
   /* do we need to open the logfile? */
-  if (_dmalloc_logpath != NULL && outfile_fd < 0) {
+  if (dmalloc_logpath != NULL && outfile_fd < 0) {
     _dmalloc_open_log();
   }
   
 #if HAVE_TIME
 #if LOG_TIME_NUMBER
-  str_p += loc_snprintf(str_p, bounds_p - str_p, "%lu: ",
-			(unsigned long)time(NULL));
+  {
+    long	now;
+    now = time(NULL);
+    str_p += loc_snprintf(str_p, bounds_p - str_p, "%ld: ", now);
+  }
 #endif /* LOG_TIME_NUMBER */
 #if HAVE_CTIME
 #if LOG_CTIME_STRING
@@ -300,10 +555,19 @@ void	_dmalloc_vmessage(const char *format, va_list args)
 #endif /* HAVE_CTIME */
 #endif /* HAVE_TIME */
   
-#if LOG_ITERATION_COUNT
+#if LOG_ITERATION
   /* add the iteration number */
   str_p += loc_snprintf(str_p, bounds_p - str_p, "%lu: ", _dmalloc_iter_c);
 #endif
+#if LOG_PID && HAVE_GETPID
+  {
+    /* we make it long in case it's big and we hope it will promote if not */
+    long	our_pid = getpid();
+    
+    /* add the pid to the log file */
+    str_p += loc_snprintf(str_p, bounds_p - str_p, "p%ld: ", our_pid);
+  }
+#endif
   
   /*
    * NOTE: the following code, as well as the function definition
@@ -325,38 +589,38 @@ void	_dmalloc_vmessage(const char *format, va_list args)
     *str_p++ = '\n';
     *str_p = '\0';
   }
-  len = str_p - str;
+  len = str_p - message_str;
   
   /* do we need to write the message to the logfile */
-  if (_dmalloc_logpath != NULL) {
-    (void)write(outfile_fd, str, len);
+  if (dmalloc_logpath != NULL) {
+    (void)write(outfile_fd, message_str, len);
   }
   
   /* do we need to print the message? */
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_MESSAGES)) {
-    (void)write(STDERR, str, len);
+    (void)write(STDERR, message_str, len);
   }
 }
 
 /*
- * message writer with printf like arguments
- */
-void	_dmalloc_message(const char *format, ...)
-  /* __attribute__ ((format (printf, 1, 2))) */
-{
-  va_list	args;
-  
-  va_start(args, format);
-  _dmalloc_vmessage(format, args);
-  va_end(args);
-}
-
-/*
- * kill the program because of an internal malloc error
+ * void _dmalloc_die
+ *
+ * DESCRIPTION:
+ *
+ * Kill the program because of an internal malloc error.
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * silent_b -> Set to 1 to not drop log entries.
  */
 void	_dmalloc_die(const int silent_b)
 {
-  char	str[1024], *stop_str;
+  char	*stop_str;
+  int	len;
   
   if (! silent_b) {
     if (BIT_IS_SET(_dmalloc_flags, DEBUG_ERROR_ABORT)) {
@@ -367,14 +631,15 @@ void	_dmalloc_die(const int silent_b)
     }
     
     /* print a message that we are going down */
-    (void)loc_snprintf(str, sizeof(str),
+    len = loc_snprintf(error_str, sizeof(error_str),
 		       "debug-malloc library: %s program, fatal error\r\n",
 		       stop_str);
-    (void)write(STDERR, str, strlen(str));
+    (void)write(STDERR, error_str, len);
     if (dmalloc_errno != ERROR_NONE) {
-      (void)loc_snprintf(str, sizeof(str), "   Error: %s (err %d)\r\n",
-			 _dmalloc_strerror(dmalloc_errno), dmalloc_errno);
-      (void)write(STDERR, str, strlen(str));
+      len = loc_snprintf(error_str, sizeof(error_str),
+			 "   Error: %s (err %d)\r\n",
+			 dmalloc_strerror(dmalloc_errno), dmalloc_errno);
+      (void)write(STDERR, error_str, len);
     }
   }
   
@@ -403,13 +668,24 @@ void	_dmalloc_die(const int silent_b)
 }
 
 /*
- * handler of error codes from procedure FUNC.  the procedure should
- * have set the errno already.
+ * void dmalloc_error
+ *
+ * DESCRIPTION:
+ *
+ * Handler of error codes.  The caller should have set the errno already
+ *
+ * RETURNS:
+ *
+ * None.
+ *
+ * ARGUMENTS:
+ *
+ * func -> Function name for the logs.
  */
 void	dmalloc_error(const char *func)
 {
   /* do we need to log or print the error? */
-  if (_dmalloc_logpath != NULL
+  if (dmalloc_logpath != NULL
       || BIT_IS_SET(_dmalloc_flags, DEBUG_PRINT_MESSAGES)) {
     
     /* default str value */
@@ -418,8 +694,8 @@ void	dmalloc_error(const char *func)
     }
     
     /* print the malloc error message */
-    _dmalloc_message("ERROR: %s: %s (err %d)",
-		     func, _dmalloc_strerror(dmalloc_errno), dmalloc_errno);
+    dmalloc_message("ERROR: %s: %s (err %d)",
+		    func, dmalloc_strerror(dmalloc_errno), dmalloc_errno);
   }
   
   /* do I need to abort? */
