@@ -1,5 +1,5 @@
 /*
- * version string for the library
+ * Version string for the library
  *
  * Copyright 2000 by Gray Watson
  *
@@ -18,7 +18,7 @@
  *
  * The author may be contacted via http://dmalloc.com/
  *
- * $Id: version.h,v 1.106.2.1 2001/02/26 17:18:55 gray Exp $
+ * $Id: version.h,v 1.120 2003/09/12 05:55:57 gray Exp $
  */
 
 #ifndef __VERSION_H__
@@ -31,15 +31,16 @@
  *	Changlog
  *	NEWS
  *	RELEASE.html
- *	Makefile.bsd.ports
+ *	configure.ac (in AC_INIT)
  *	dmalloc.spec
- *	dmalloc.texi (2 places)
- *	dmalloc.h.3 (DMALLOC_VERSION defines at top of file)
+ *	dmalloc.texi
+ *	dmalloc.h.4 (DMALLOC_VERSION defines at top of file)
+ *      ports/.../Makefile and ports/.../distfile files
  *
- * Make sure to also cvs tag the release.  dmalloc_release_X_X_X[_bX]
+ * Make sure to also cvs tag the release.  dmalloc_release_X_X_X
  */
-static	char	*dmalloc_version = "4.8.2" ;
+static	char	*dmalloc_version = "5.2.3" ;
 
-/* Version Date: $Date: 2001/02/26 17:18:55 $ */
+/* Version Date: $Date: 2003/09/12 05:55:57 $ */
 
 #endif /* ! __VERSION_H__ */
